import { SoundcraftUI } from 'soundcraft-ui-connection';
import $ from 'jquery';
import '@fontsource/open-sans';
import './index.css';

const HOST = '192.168.0.111';
const soundcraft = new SoundcraftUI(HOST);
const app = $('#app');
const statusText = $('#status');

// https://github.com/fmalcher/soundcraft-ui/tree/main/packages/mixer-connection

statusText.text('ÜHENDAMAS');
soundcraft.connect();

const channels = [
  {
    name: ['MIC 1', 'MIC 2'],
    channel: [1, 2],
    separate: true,
    bothBtn: false,
    plural: 'Mikrofonid'
  },
  {
    name: ['Lava', 'HDMI'],
    channel: [5, 6],
    separate: true,
    bothBtn: false,
  },
];

const faders = [
  {
    name: ['Lava', 'HDMI'],
    channel: [5, 6],
  }
];

const subscriptions: any[] = [];

function cleanUp(): void {
  app.empty();
  subscriptions.forEach((sub) => sub.unsubscribe());
  subscriptions.splice(0, subscriptions.length);
}

soundcraft.status$.subscribe((status) => {
  console.log(status);
  if (status.type !== 'OPEN') {
    statusText.text('ÜHENDUSETA');
    statusText.addClass('disconnected');
    return;
  }
  statusText.text('ÜHENDUSES');
  statusText.removeClass('disconnected');
  cleanUp();
  channels.forEach((channel) => {
    const plural = (channel.plural ||
      (Array.isArray(channel.name) ? channel.name.join(' ja ') : 'kõik'));
    const managers = channel.channel.map((num) => soundcraft.master.input(num));
    const statuses = channel.channel.map(() => false);
    const wrapper = $('<div>').addClass('channel section');
    const statusText = $('<div>').addClass('status');
    const btnGroup = $('<div>').addClass('btn-group');
    const mute = $('<button>').text('välja');
    const unmute = $('<button>').text('sisse');
    // wrapper.append($('<span>').text(channel.name).addClass('title'));
    if (channel.separate) {
      wrapper.addClass('separated');
      channel.channel.forEach((num, index) => {
        const name = Array.isArray(channel.name) ? channel.name[index] : index;
        const ibtnGroup = $('<div>').addClass('btn-group').addClass('btn-' + name.toString().toLowerCase());
        const imute = $('<button>').text(`${name} välja`.toUpperCase());
        const iunmute = $('<button>').text(`${name} sisse`.toUpperCase());
        imute.on('click', () => managers[index].mute());
        iunmute.on('click', () => managers[index].unmute());
        ibtnGroup.append(iunmute, imute);
        wrapper.append(ibtnGroup);
      });
      mute.text('mõlemad välja');
      unmute.text('mõlemad sisse');
    }
    channel.channel.forEach((num, index) => {
      const sub = managers[index].mute$.subscribe((status) => {
        statuses[index] = status === 1;
        let text = '';
        if (channel.separate) {
          if (statuses.every((stat) => stat === true)) {
            text = plural + ' on väljas.';
          } else {
            const off = statuses.map(
              (stat, index) => stat ? index : false
            ).filter((stat) => stat !== false) as number[];
            if (!off.length) {
              text = plural + ' on sees.';
            } else {
              text = off.map((indx) => channel.name[indx]).join(' ja ') + ' on väljas.';
            }
          }
        } else {
          text = status ? 'väljas' : 'sees';
        }
        statusText.text(text);
      });
      subscriptions.push(sub);
    });
    if (channel.bothBtn !== false) {
      mute.on('click', () => managers.forEach((manager) => manager.mute()));
      unmute.on('click', () => managers.forEach((manager) => manager.unmute()));
      btnGroup.append(unmute, mute);
      wrapper.append(btnGroup);
    }
    wrapper.append(statusText);
    app.append(wrapper);
  });

  faders.forEach((channel) => {
    const managers = channel.channel.map((num) => soundcraft.master.input(num));
    const wrapper = $('<div>').addClass('fader-group section');
    channel.channel.forEach((num, index) => {
      const fader = $('<div>').addClass('fader');
      const nameText = $('<div>').addClass('title');
      const statusText = $('<span>').addClass('volume');
      const input = $('<input>').attr('min', 0).attr('max', 1).attr('step', 0.01).attr('type', 'range');
      input.on('input', () => {
         managers[index].setFaderLevel(input.val() as number);
      });
      const sub = managers[index].faderLevel$.subscribe((level) => {
        statusText.text(`${Math.floor(level * 100)}%`);
        input.val(level);
      });
      nameText.text((Array.isArray(channel.name) ? channel.name[index] : index) + ' helireguleerimine');
      fader.append(nameText, statusText, input);
      wrapper.append(fader);
      subscriptions.push(sub);
    });
    app.append(wrapper);
  });
});
